import net.sf.emff.Track;

import net.sf.flashlib.util.Delegate;

/**
 * A playlist with tracks.
 * 
 * @version 0.0.1
 * @author Marc Reichelt
 */
class net.sf.emff.Playlist {
	
	/**
	 * Tracks of this playlist.
	 */
	private var tracks : Array = null;
	
	/**
	 * XML data for external playlist.
	 */
	private var xml : XML = null;
	
	/**
	 * Indicates whether the playlist is mixed.
	 */
	private var shuffle : Boolean = false;
	
	/**
	 * Indicates whether this playlist is initialized.
	 */
	private var initialized : Boolean = false;
	
	
	
	/**
	 * Creates a new playlist with the given parameters.
	 * 
	 * @param src URLs (separated by commas) or <code>null</code> if no URLs are given.
	 * @param playlist URL to external playlist or <code>null</code> if no external
	 *                  playlist is given.
	 * @param shuffle Indicates whether the playlist should be mixed after creation
	 *                 (<code>true</code>) or not (<code>false</code>).
	 */
	public function Playlist(src : String, playlist : String, shuffle : Boolean) {
		
		this.shuffle = shuffle;
		
		tracks = new Array();
		
		if (playlist != null) {
			// external playlist file
			loadExternalPlaylist(playlist);
			return;
		} else {
			if (src != null && src != "") {
				// URLs given by src attribute, seperated by commas
				var urls : Array = src.split(",");
				
				for (var i : Number = 0; i < urls.length; i++) {
					var url : String = urls[i];
					tracks.push(new Track(url));
				}
				
				if (shuffle) {
					mix();
				}
			} else {
				// no playlist data given, set default track
				tracks.push(new Track(null));
			}
		}
		
		// no external playlist - initialized
		onInit();
	}
	
	
	
	/**
	 * TODO: Docs
	 */
	public function onInit() : Void {
		initialized = true;
	}
	
	/**
	 * Sorts the tracks in a random order.
	 */
	private function mix() : Void {
		var tmp : Object, rand : Number;
		for (var i : Number = 0; i < tracks.length; i++) {
			rand = Math.floor(Math.random() * tracks.length);
			tmp = tracks[i];
			tracks[i] = tracks[rand];
			tracks[rand] = tmp;
		}
	 }
	
	/**
	 * Get length of this playlist.
	 * 
	 * @return Length of this playlist.
	 */
	public function getLength() : Number {
		return tracks.length;
	}
	
	/**
	 * Get track for the given index.
	 * 
	 * @param index The index for the track.
	 * @return A <code>Track</code> if there is a valid track according to the given
	 *          index, <code>null</code> otherwise.
	 */
	public function getTrack(index : Number) : Track {
		if (index != null && index >= 0 && index < tracks.length) {
			return Track(tracks[index]);
		} else {
			return null;
		}
	}
	
	/**
	 * Creates a playlist by loading a XSPF from the specified file.
	 * 
	 * @param xspfFile URL to XSPF file.
	 */
	private function loadExternalPlaylist(xspfFile : String) : Void {
		xml = new XML();
		xml.ignoreWhite = true;
		xml.onLoad = Delegate.create(this, onLoadXML);
		xml.load(xspfFile);
	}
	
	private function onLoadXML(success : Boolean) : Void {
		if (success) {
			// xml data ok
			var root : XMLNode = xml.firstChild;
			
			if (root.nodeName == "playlist") {
				var items : Array = root.childNodes;
				
				// look for <trackList>
				for (var i : Number = 0; i < items.length; i++) {
					
					// trackList found, parse tracks
					var item : XMLNode = items[i];
					if (item.nodeName == "trackList") {
						var tracks : Array = item.childNodes;
						for (var j : Number = 0; j < tracks.length; j++) {
							var track : XMLNode = tracks[j];
							if (track.nodeName == "track") {
								
								var location : String = null;
								var title : String = null;
								var creator : String = null;
								var annotation : String = null;
								var info : String = null;
								var image : String = null;
								var album : String = null;
								var trackNum : Number = null;
								var duration : Number = null;
								
								var trackItems : Array = track.childNodes;
								for (var k : Number = 0; k < trackItems.length; k++) {
									var trackItem : XMLNode = trackItems[k];
									var value : String = trackItem.firstChild.nodeValue;
									switch (trackItem.nodeName) {
									case "location":
										location = value;
										break;
									case "title":
										title = value;
										break;
									case "creator":
										creator = value;
										break;
									case "annotation":
										annotation = value;
										break;
									case "info":
										info = value;
										break;
									case "image":
										image = value;
										break;
									case "album":
										album = value;
										break;
									case "trackNum":
										trackNum = Number(value);
										break;
									case "duration":
										duration = Number(value);
										break;
									}
								}
								
								// parsing this track done
								// location must not be null
								if (location != null) {
									var newTrack : Track = new Track(location);
									newTrack.setTitle(title);
									newTrack.setCreator(creator);
									newTrack.setAnnotation(annotation);
									newTrack.setInfo(info);
									newTrack.setImage(image);
									newTrack.setAlbum(album);
									newTrack.setTrackNum(trackNum);
									newTrack.setDuration(duration);
									
									this.tracks.push(newTrack);
								}
							}
						}
					}
					
					if (this.tracks.length == 0) {
						this.tracks.push(new Track(null));
					}
					
					if (shuffle) {
						mix();
					}
					
					onInit();
					return;
				}
			}
		}
		
		// loading failed
		tracks.push(new Track(null));
		onInit();
	}
	
	/**
	 * Get a string representation for this track.
	 * 
	 * @result String representation for this track.
	 */
	public function toString() : String {
		var result : String = "Playlist {\n";
		
		for (var i : Number = 0; i < tracks.length; i++) {
			result += "  " + Track(tracks[i]).toString() + "\n";
		}
		
		result += "}";
		
		return result;
	}
	
	/**
	 * Indicates whether this playlist is initialized.
	 * 
	 * @return <code>true</code> if this playlist is initialized,
	 *          <code>false</code> otherwise.
	 */
	public function isInitialized() : Boolean {
		return initialized;
	}
	
}
